<?php

// Ve vyzadovanem souboru jsou ulozeny udaje nutne k pripojeni se k databazi.  
require('pit_con.php');

// Trida reprezentujici vrstvu Model architektury Model-View-Controller (MVC).
// Trida nema zadne atributy.
class Predmety_db
{

  // Funkce, ktera do asociativniho pole $chyba ulozi informace o nastale chybe.
  // Parametr $zdroj udava pripadny zdroj chyby.
  public function zjisti_chybu($zdroj = false)
  {
    // Jestlize parametr $zdroj <> FALSE, vola se funkce oci_error s parametrem, jinak bez parametru.
    $chyba = $zdroj ? oci_error($zdroj) : oci_error();
		
	return($chyba);
  }

  // Funkce, ktera otevre spojeni s databazi. Udaje pro pripojeni jsou ulozeny v konstantach 
  // ORACLE_LOGIN, ORACLE_HESLO a ORACLE_DB.
  public function navaz_spojeni()
  {
    // Posledni parametr udava, ze na webovem serveru je pouzito kodovani Windows 1250, tak jak mu rozumi Oracle.
    $spojeni = oci_connect(ORACLE_LOGIN, ORACLE_HESLO, ORACLE_DB, 'EE8MSWIN1250');
	
	return ($spojeni);
  }
  
  // Funkce, ktera pripravi dotaz pro serazeny vypis obsahu tabulky PREDMETY podle nazvu predmetu.
  // Parametr $spojeni udava otevrene spojeni s databazi.
  public function priprav_select($spojeni) 
  {
    // Prikaz v SQL, ktery chceme vykonat v databazi. 
    $prikaz_sql = "SELECT zkratka, nazev, kredity, hodin_pr, hodin_cv".
	              "  FROM predmety".
			      " ORDER BY nazev";
    
	// Navazani pripraveneho SQL prikazu s otevrenym spojenim do databaze.
	$prikaz = oci_parse($spojeni, $prikaz_sql);
	
	return($prikaz);	
  }
  
  // Funkce, ktera pripravi vlozeni noveho predmetu do tabulky PREDMETY. 
  // Parametr $spojeni udava otevrene spojeni s databazi.
  // Ostatni parametry jsou polozky vkladaneho predmetu.
  public function priprav_insert($spojeni, $zkratka, $nazev, $kredity, $hodin_pr, $hodin_cv)
  {
    // Prikaz v SQL s parametry, ktery chceme vykonat v databazi.
    $prikaz_sql = "INSERT INTO predmety (zkratka, nazev, kredity, hodin_pr, hodin_cv) ".
	              "VALUES (:zkratka, :nazev, :kredity, :hodin_pr, :hodin_cv)";

	// Navazani pripraveneho SQL prikazu s otevrenym spojenim do databaze.
	$prikaz = oci_parse($spojeni, $prikaz_sql);
	
	// Predani hodnot parametrum SQL prikazu INSERT.
	oci_bind_by_name($prikaz, ':zkratka', $zkratka);
	oci_bind_by_name($prikaz, ':nazev', $nazev);
	oci_bind_by_name($prikaz, ':kredity', $kredity);
	oci_bind_by_name($prikaz, ':hodin_pr', $hodin_pr);
	oci_bind_by_name($prikaz, ':hodin_cv', $hodin_cv);
		
	return($prikaz);
  }
  
  // Funkce, ktera pripravi zmenu (aktualizaci) predmetu, ktery ma zkratku $zkratka - zkratku nelze menit.
  // Parametr $spojeni udava otevrene spojeni s databazi.
  // Parametr $zkratka obsahuje zkratku predmetu, ktery bude aktualizovan.
  // Ostatni parametry obsahuji nove hodnoty aktualizovaneho predmetu.  
  public function priprav_update($spojeni, $zkratka, $nazev, $kredity, $hodin_pr, $hodin_cv)
  {
    // Prikaz v SQL s parametry, ktery chceme vykonat v databazi.
    $prikaz_sql = "UPDATE predmety SET nazev = :nazev, kredity = :kredity, hodin_pr = :hodin_pr, hodin_cv = :hodin_cv ".
	              " WHERE zkratka = :zkratka";
	
	// Navazani pripraveneho SQL prikazu s otevrenym spojenim do databaze.
	$prikaz = oci_parse($spojeni, $prikaz_sql);
	
	// Predani hodnot parametrum SQL prikazu UPDATE.
	oci_bind_by_name($prikaz, ':zkratka', $zkratka);
	oci_bind_by_name($prikaz, ':nazev', $nazev);
	oci_bind_by_name($prikaz, ':kredity', $kredity);
	oci_bind_by_name($prikaz, ':hodin_pr', $hodin_pr);
	oci_bind_by_name($prikaz, ':hodin_cv', $hodin_cv);
		
	return($prikaz);				  
  }
  
  // Funkce, ktera pripravi zruseni (smazani) predmetu, ktery ma zkratku $zkratka.
  // Parametr $spojeni udava otevrene spojeni s databazi.
  // Parametr $zkratka obsahuje zkratku predmetu, ktery bude smazan.  
  public function priprav_delete($spojeni, $zkratka)
  {
    // Prikaz v SQL s parametry, ktery chceme vykonat v databazi.
    $prikaz_sql = "DELETE FROM predmety ".
	              " WHERE zkratka = :zkratka";

	// Navazani pripraveneho SQL prikazu s otevrenym spojenim do databaze.
	$prikaz = oci_parse($spojeni, $prikaz_sql);
	
	// Predani hodnot parametrum SQL prikazu DELETE.
	oci_bind_by_name($prikaz, ':zkratka', $zkratka);
	
	return($prikaz);				  	
  }
  
  // Funkce, ktera vvykona pripraveny prikaz SELECT v databazi.
  // Parametr $prikaz obsahuje informaci, jaky prikaz a ve ktere databazi se ma vykonat.
  public function ziskej_data($prikaz) 
  {	
	// Navratova hodnota urcuje, zda vse probehlo v poradku nebo se vyskytla chyba (hodnota FALSE).
	$vysledek = oci_execute($prikaz, OCI_NO_AUTO_COMMIT);
	
	return($vysledek);
  }
  
  // Funkce, ktera vykona pripraveny prikaz (INSERT, UPDATE, DELETE) v databazi.
  // Parametr $prikaz obsahuje informaci, jaky prikaz a ve ktere databazi se ma vykonat.
  public function zpracuj_data($prikaz)
  {
    // Navratova hodnota urcuje, zda vse probehlo v poradku nebo se vyskytla chyba (hodnota FALSE).
    $vysledek = oci_execute($prikaz, OCI_COMMIT_ON_SUCCESS);
	
	return($vysledek);	
  }
  
  // Funkce, ktera uvolni zabrane programove prostredky pripravenym prikazem, ktery jiz byl v databazi vykonan.
  // Parametr $prikaz obsahuje informaci, ktery ma uvolnit svoje programove prostredky.
  public function uvolni_prikaz($prikaz) 
  {  
    // Navratova hodnota urcuje, zda vse probehlo v poradku nebo se vyskytla chyba (hodnota FALSE).
	$vysledek = oci_free_statement($prikaz);
	
    return($vysledek);
  }
  
  // Funkce, ktera ukonci aktivni spojeni s databazi.
  // Parametr $spojeni udava otevrene spojeni s databazi, ktere bude uzavreno.
  public function uzavri_spojeni($spojeni) {
  
    // Navratova hodnota urcuje, zda vse probehlo v poradku nebo se vyskytla chyba (hodnota FALSE).
	$vysledek = oci_close($spojeni);
	
	return($vysledek);
  }   
}

// Trida reprezentujici vrstvu View architektury Model-View-Controller (MVC).
// Trida nema zadne atributy.
class Predmety_web
{
  // Procedura, ktera generuje zahlavi dynamicke XHTML stranky pro evidenci predmetu.
  public function zahlavi()
  {
    header('Content-Type: text/html; charset=windows-1250');
	
	echo '<?xml version="1.0" encoding="windows-1250"?>'."\n";
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'."\n";
	echo '<html xmlns = "http://www.w3.org/1999/xhtml" xml:lang = "en">'."\n";
	echo '<head>'."\n";
	echo '<title>Evidence pedmt</title>'."\n";
	echo '<link rel="stylesheet" type="text/css" href="style.css" />'."\n";
	echo '</head>'."\n";
	echo '<body>'."\n";
	echo '<h1>Evidence pedmt</h1>'."\n";
  }
  
  // Procedura, ktera generuje paticku dynamicke XHTML stranky pro evidenci predmetu.
  public function paticka()
  {
    echo '<h4>'."\n";
	echo date('j.n.Y G:i:s')."\n";
	echo '</h4>'."\n";
	
    echo '</body>'."\n";
	echo '</html>'."\n";
  }

  // Procedura, ktera zobrazi uplnou informaci o posledni detekovane chybe.
  public function zobraz_chybu($chyba, $radek, $soubor) 
  {    
	echo '<p><em>Chyba</em>: na dku '.$radek.' v souboru '.$soubor.'</p>';
	echo '<pre>'.$chyba['message'].'</pre>';
	
	if($chyba['offset'] > 0)
	{
	  echo '<p><em>Chyba</em>: na pozici '.$chyba['offset'].' v pkazu:</p>';
	  echo '<pre>'.$chyba['sqltext'].'</pre>';	  
	}
	
	exit;
  }
  
  // Procedura, ktera generuje XHTML kod tabulky, ve ktere jsou zobrazeny vsechna data z tabulky PREDMETY.
  public function zobraz_predmety($prikaz)
  {
    echo '<h3>'."\n";
	echo "Evidovan pedmty:\n";
	echo '</h3>'."\n";
  	
	// Hlavicka zobrazovane tabulky.
    echo '<table border = "1">'."\n";
	echo '<tr>'."\n";
	echo '<th>Zkratka</th>'."\n";
	echo '<th>Nzev</th>'."\n";
	echo '<th>Kredity</th>'."\n";
	echo '<th>Hodin pednek</th>'."\n";
	echo '<th>Hodin cvien</th>'."\n";
	echo '<th>Akce</th>'."\n";
	echo '</tr>'."\n";
    
	// Zobraz dalsi predmet, pokud je odpovedi na pripraveny dotaz.
	while (oci_fetch($prikaz)) 
	{
	  echo '<tr>'."\n";
	 	  
      echo '<td>'.oci_result($prikaz, "ZKRATKA").'</td>'."\n";  // hodnota sloupce ZKRATKA
      echo '<td>'.oci_result($prikaz, "NAZEV").'</td>'."\n";    // hodnota sloupce NAZEV
	  echo '<td>'.oci_result($prikaz, "KREDITY").'</td>'."\n";  // hodnota sloupce KREDITY
	  echo '<td>'.oci_result($prikaz, "HODIN_PR").'</td>'."\n"; // hodnota sloupce HODIN_PR
	  echo '<td>'.oci_result($prikaz, "HODIN_CV").'</td>'."\n"; // hodnota sloupce HODIN_CV
	  	  
	  // U kazdeho predmetu zobrazime jeste dve tlacitka pro smazani a aktualizaci predmetu, jsou soucasti formulare.
	  echo '<td>'."\n";	  	  
	  echo '<form action="pit.php" method="post">'."\n";
	  echo '<div>'."\n";	  	  	  
	  echo '<input type="hidden" name="zkratka" value="'.oci_result($prikaz, "ZKRATKA").'" />'."\n";
	  echo '<input type="hidden" name="nazev" value="'.oci_result($prikaz, "NAZEV").'" />'."\n";
	  echo '<input type="hidden" name="kredity" value="'.oci_result($prikaz, "KREDITY").'" />'."\n";
	  echo '<input type="hidden" name="hodin_pr" value="'.oci_result($prikaz, "HODIN_PR").'" />'."\n";
	  echo '<input type="hidden" name="hodin_cv" value="'.oci_result($prikaz, "HODIN_CV").'" />'."\n";	  	 
	  echo '<input type="submit" name="smaz" value="Sma" />'."\n";
	  echo '<input type="submit" name="zmen" value="Zm" />'."\n";	  
	  echo '</div>'."\n";	  	  	  	  
	  echo '</form>'."\n";
	  echo '</td>'."\n";
	  	  
      echo '</tr>'."\n";	  
    }        
    echo '</table>'."\n";  
  }
  
  // Procedura, ktera zobrazi formular pro zadani noveho predmetu.
  public function novy_predmet()
  {
    echo '<h3>'."\n";
	echo "Nov pedmt:\n";
	echo '</h3>'."\n";
  
    echo '<form action="pit.php" method="post">'."\n";

    echo '<table border = "1">'."\n";
	echo '<tr>'."\n";
	echo '<th>Zkratka</th>'."\n";
	echo '<th>Nzev</th>'."\n";
	echo '<th>Kredity</th>'."\n";
	echo '<th>Hodin pednek</th>'."\n";
	echo '<th>Hodin cvien</th>'."\n";
	echo '<th>Akce</th>'."\n";
	echo '</tr>'."\n";
    echo '<tr>'."\n";  
	echo '<td><input type="text" name="zkratka" size="10" /></td>'."\n";
	echo '<td><input type="text" name="nazev" size="50" /></td>'."\n";
	echo '<td><input type="text" name="kredity" size="2" /></td>'."\n";
	echo '<td><input type="text" name="hodin_pr" size="2" /></td>'."\n";
	echo '<td><input type="text" name="hodin_cv" size="2" /></td>'."\n";
	echo '<td><input type="submit" name="vloz" value="Vlo" /></td>'."\n";
	echo '</tr>'."\n";		
    echo '</table>'."\n";  	
	
	echo '</form>'."\n";
  }
  
   // Procedura, ktera zobrazi formular pro aktualizaci existujiciho predmetu.
   public function zmen_predmet()
   {
    echo '<h3>'."\n";
	echo "Editace pedmtu:\n";
	echo '</h3>'."\n";
   
    echo '<form action="pit.php" method="post">'."\n";

    echo '<table border = "1">'."\n";
	echo '<tr>'."\n";
	echo '<th>Zkratka</th>'."\n";
	echo '<th>Nzev</th>'."\n";
	echo '<th>Kredity</th>'."\n";
	echo '<th>Hodin pednek</th>'."\n";
	echo '<th>Hodin cvien</th>'."\n";
	echo '<th>Akce</th>'."\n";
	echo '</tr>'."\n";  
    echo '<tr>'."\n";  
	echo '<td>'."\n";
	echo $_POST["zkratka"]."\n";
	echo '<input type="hidden" name="zkratka" value="'.$_POST["zkratka"].'" />'."\n";
	echo '</td>'."\n";
	echo '<td><input type="text" name="nazev" size="50" value="'.$_POST["nazev"].'" /></td>'."\n";
	echo '<td><input type="text" name="kredity" size="2" value="'.$_POST["kredity"].'" /></td>'."\n";
	echo '<td><input type="text" name="hodin_pr" size="2" value="'.$_POST["hodin_pr"].'" /></td>'."\n";
	echo '<td><input type="text" name="hodin_cv" size="2" value="'.$_POST["hodin_cv"].'" /></td>'."\n";
	echo '<td>'."\n";
	echo '<input type="submit" name="uloz" value="Ulo" />'."\n";
	echo '<input type="reset" name="reset" value="Reset" />'."\n";
	echo '</td>'."\n";
	echo '</tr>'."\n";		
    echo '</table>'."\n";  	
	
	echo '</form>'."\n";
     
   }
}

// Trida reprezentujici vrstvu Controller architektury Model-View-Controller (MVC).
// Trida ma dva atributy:
// $predmety_db - instance tridy Predmety_db   - vrstva Model
// $predmety_web - instance tridy Predmety_web - vrstva View
class Predmety 
{  
  private $predmety_db = null;
  private $predmety_web = null;

  // Konstruktor tridy Predmety, vytvori instanci tridy Predmety_db a instanci tridy Predmety_web.
  public function __construct()
  {
    $this->predmety_db = new Predmety_db();
	$this->predmety_web = new Predmety_web();
  }
  
  // Procedura, ktera vygeneruje XHTML kod oznamujici, jaka chyba nastala.
  public function chyba($zdroj = false) 
  {
	$chyba = $this->predmety_db->zjisti_chybu($zdroj);
	$this->predmety_web->zobraz_chybu($chyba, __LINE__, __FILE__);
  }
  
  // Procedura, ktera reaguje na akcni tlacitka formularu a podle vybrane akce meni obsah tabulky PREDMETY.
  public function zmena_dat($spojeni)
  {
    // Stisknuto tlacitko pro vlozeni noveho predmetu.
	if(isset($_POST["vloz"]))
	{
	  // Priprav prikaz INSERT pro vlozeni noveho predmetu.
	  $prikaz = $this->predmety_db->priprav_insert($spojeni, $_POST["zkratka"], $_POST["nazev"], $_POST["kredity"], $_POST["hodin_pr"], $_POST["hodin_cv"]);
	}
	// Stisknuto tlacitko pro ulozeni zmen vybraneho predmetu.
	elseif(isset($_POST["uloz"]))
	{
	  // Priprav prikaz UPDATE pro ulozeni zmen vybraneho predmetu.
	  $prikaz = $this->predmety_db->priprav_update($spojeni, $_POST["zkratka"], $_POST["nazev"], $_POST["kredity"], $_POST["hodin_pr"], $_POST["hodin_cv"]);
	}
	// Stisknuto tlacitko pro smazani vybraneho predmetu.
	elseif(isset($_POST["smaz"]))
	{
	  // Priprav prikaz DELETE pro smazani vybraneho predmetu.
	  $prikaz = $this->predmety_db->priprav_delete($spojeni, $_POST["zkratka"]);
	}
    
	if(isset($_POST["vloz"]) or isset($_POST["uloz"]) or isset($_POST["smaz"]))
	{
	  // Behem pripravy SQL prikazu muze dojit k chybe.
	  if($prikaz == FALSE)
	  {
	    $this->chyba($spojeni);
	  }
	  
	  // Vykonani SQL prikazu muze vyvolat chybu.
	  $vysledek = $this->predmety_db->zpracuj_data($prikaz);
	  if($vysledek == FALSE)
	  {
	    $this->chyba($prikaz);
	  }
	  
	  // Po vykonani SQL prikazu je slusne vratit zabrane programove prostredky.
	  $vysledek = $this->predmety_db->uvolni_prikaz($prikaz);
	  if($vysledek == FALSE)
	  {
	    $this->chyba($prikaz);
	  }
	}
  }
  
  // Procedura, ktera zobrazi v prehledne XHTML tabulce obsah tabulky PREDMETY.
  public function zobraz_data($spojeni)
  {
    // Priprav prikaz SELECT pro nacteni seznamu predmetu.
	$prikaz = $this->predmety_db->priprav_select($spojeni);
	if($prikaz == FALSE)
	{
	  $this->chyba($spojeni);
	}
	
	// Vykonani prikazu SELECT muze vyvolat chybu.
	$vysledek = $this->predmety_db->ziskej_data($prikaz);	
	if($vysledek == FALSE)
	{
	  $this->chyba($prikaz);
    }
    
	// Zobrazeni tabulky PREDMETY v XHTML tabulce.
	$this->predmety_web->zobraz_predmety($prikaz);

	// Po vykonani SQL prikazu je slusne vratit zabrane programove prostredky.
	$vysledek = $this->predmety_db->uvolni_prikaz($prikaz);
	if($vysledek == FALSE)
	{
	  $this->chyba($prikaz);	
	}
  }
  
  // Procedura, ktera zobrazuje formular, ato bud pro vlozeni noveho predmetu, nebo zmenu vybraneho.
  public function formular()
  {
    // Jestlize je stisknuto tlacitko pro zmenu vybraneho predmetu
	if(isset($_POST["zmen"]))
	{
	  // zobraz formular pro aktualizaci predmetu,
	  $this->predmety_web->zmen_predmet();
	}
	else
	{
	  // jinak zobraz formular pro vlozeni noveho predmetu.
	  $this->predmety_web->novy_predmet();
	}    
  }
  
  // Procedura zastupujici "hlavni" program, generuje celou XHTML stranku a komunikuje s databazi.
  public function predmety()
  {
	// Generuje zacatek XHTML stranky.
	$this->predmety_web->zahlavi();
	
	// Navaz spojeni s databazi.
	$spojeni = $this->predmety_db->navaz_spojeni();
	if($spojeni == FALSE)
	{
      $this->chyba();
	}
	
	$this->zmena_dat($spojeni);    // Proved zmenu dat, pokud je zadana.
	$this->zobraz_data($spojeni);  // Zobraz obsah tabulky PREDMETY v XHTML tabulce
	
	// Ukonci spojeni s databazi.
	$vysledek = $this->predmety_db->uzavri_spojeni($spojeni);
	if($vysledek == FALSE)
	{
      $this->chyba();
	}
	
	// Pod tabulkou predmetu zobraz formular pro pridani/aktualizaci predmetu.
	$this->formular();
	
	// Generuj konec XHTML stranky.
	$this->predmety_web->paticka();	
  }
}

$predmety = new Predmety();

$predmety->predmety();
?>
